// Copyright (C) 2011-2013 R M Yorston
// Licence: GPLv2+

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Clutter = imports.gi.Clutter;
const EndSessionDialog = imports.ui.endSessionDialog;
const Main = imports.ui.main;
const UserMenu = imports.ui.userMenu;

const _f = imports.gettext.domain('frippery-shut-down-menu').gettext;

// Need to reimplement here the missing bits from LoginManager
// Stolen from alternative status menu extension

function loginManager_hibernate() {
    if (this._proxy) {
        // systemd path
        this._proxy.call("Hibernate",
                         GLib.Variant.new('(b)', [true]),
                         Gio.DBusCallFlags.NONE,
                         -1, null, null);
    } else {
        // Can't do in ConsoleKit
        this.emit('prepare-for-sleep', true);
        this.emit('prepare-for-sleep', false);
    }
}

function loginManager_canHibernate(asyncCallback) {
    if (this._proxy) {
        // systemd path
        this._proxy.call("CanHibernate",
                         null,
                         Gio.DBusCallFlags.NONE,
                         -1, null, function(proxy, asyncResult) {
                             let result, error;

                             try {
                                 result = proxy.call_finish(asyncResult);
                             } catch(e) {
                                 error = e;
                             }

                             if (error)
                                 asyncCallback(false);
                             else
                                 asyncCallback(result[0] != 'no');
                         });
    } else {
        Mainloop.idle_add(Lang.bind(this, function() {
            asyncCallback(false);
            return false;
        }));
    }
}

function statusMenu_updateHaveHibernate() {
    loginManager_canHibernate.call(this._loginManager, Lang.bind(this,
        function(result) {
            this._haveHibernate = result;
            this._updateSuspendOrPowerOff();
        }));
}

let myUpdateSuspendOrPowerOff, origUpdateSuspendOrPowerOff;
let myUpdateButtons, origUpdateButtons;
let shutdownDialogContent, originalDialogContent;

function init(extensionMeta) {
    let localePath = extensionMeta.path + '/locale';
    imports.gettext.bindtextdomain('frippery-shut-down-menu', localePath);

    let usermenu = Main.panel.statusArea['userMenu'];

    if ( !usermenu ) {
        return;
    }

    originalDialogContent = EndSessionDialog.DialogContent[1];
    shutdownDialogContent = {
        subject: C_("title", "Power Off"),
        inhibitedDescription: _("Click Power Off to quit these applications and power off the system."),
        uninhibitedDescription: function(seconds) {
            return ngettext(
                "The system will power off automatically in %d second.",
                "The system will power off automatically in %d seconds.",
                seconds).format(seconds);
        },
        endDescription: _("Powering off the system."),
        secondaryButtons: [{ signal: 'ConfirmedSuspend',
                           label:  _("Suspend") },
                         { signal: 'ConfirmedHibernate',
                           label:  _f("Hibernate") },
                         { signal: 'ConfirmedReboot',
                           label:  C_("button", "Restart") }],
        confirmButtons: [{ signal: 'ConfirmedShutdown',
                           label:  C_("button", "Power Off") }],
        iconName: 'system-shutdown-symbolic',
        iconStyleClass: 'end-session-dialog-shutdown-icon'
    };

    origUpdateSuspendOrPowerOff = usermenu._updateSuspendOrPowerOff;
    myUpdateSuspendOrPowerOff = function() {
        if (!this._suspendOrPowerOffItem)
            return;

        this._suspendOrPowerOffItem.actor.visible = this._haveShutdown || this._haveSuspend;

        if ( !this._haveShutdown && this._haveSuspend ) {
            this._suspendOrPowerOffItem.updateText(_("Suspend"), null);
        }
        else {
            this._suspendOrPowerOffItem.updateText(_f("Shut Down..."), null);
        }
    };

    origUpdateButtons =
        EndSessionDialog.EndSessionDialog.prototype._updateButtons;

    myUpdateButtons = function() {
        let dialogContent = EndSessionDialog.DialogContent[this._type];
        let buttons = [];
        let usermenu = Main.panel.statusArea['userMenu'];

        usermenu._updateHaveSuspend();
        statusMenu_updateHaveHibernate.call(usermenu);

        if ( dialogContent.secondaryButtons ) {
            for (let i = 0; i < dialogContent.secondaryButtons.length; i++) {
                let signal = dialogContent.secondaryButtons[i].signal;
                let label = dialogContent.secondaryButtons[i].label;

                if ( signal == 'ConfirmedHibernate' ) {
                    if ( usermenu._haveHibernate ) {
                        buttons.push({ action: Lang.bind(this, this._onHibernate),
                                       label: label });
                    }
                }
                else if ( signal == 'ConfirmedSuspend' ) {
                    if ( usermenu._haveSuspend ) {
                        buttons.push({ action: Lang.bind(this, this._onSuspend),
                                       label: label });
                    }
                }
                else {
                    buttons.push({ action: Lang.bind(this, function() {
                                           this._confirm(signal);
                                       }),
                                   label: label });
                }
            }
        }

        buttons.push({ action: Lang.bind(this, this.cancel),
                         label:  _("Cancel"),
                         key:    Clutter.Escape });

        for (let i = 0; i < dialogContent.confirmButtons.length; i++) {
            let signal = dialogContent.confirmButtons[i].signal;
            let label = dialogContent.confirmButtons[i].label;
            buttons.push({ action: Lang.bind(this, function() {
                                       this._confirm(signal);
                                   }),
                           label: label });
        }

        this.setButtons(buttons);
    };
}

function enable() {
    let usermenu = Main.panel.statusArea['userMenu'];

    if ( !usermenu ) {
        return;
    }

    EndSessionDialog.EndSessionDialog.prototype._onHibernate = function() {
        let usermenu = Main.panel.statusArea['userMenu'];

        Main.overview.hide();

        this._stopTimer();
        this._dbusImpl.emit_signal('Canceled', null);
        this.close();
        loginManager_hibernate.call(usermenu._loginManager);
    };

    EndSessionDialog.EndSessionDialog.prototype._onSuspend = function() {
        let usermenu = Main.panel.statusArea['userMenu'];

        Main.overview.hide();

        this._stopTimer();
        this._dbusImpl.emit_signal('Canceled', null);
        this.close();
        usermenu._loginManager.suspend();
    };

    EndSessionDialog.EndSessionDialog.prototype._updateButtons =
        myUpdateButtons;
    EndSessionDialog.DialogContent[1] = shutdownDialogContent;

    usermenu._updateSuspendOrPowerOff = myUpdateSuspendOrPowerOff;
    usermenu._updateHaveSuspend();
    statusMenu_updateHaveHibernate.call(usermenu);
    usermenu._updateSuspendOrPowerOff();
}

function disable() {
    let usermenu = Main.panel.statusArea['userMenu'];

    if ( !usermenu ) {
        return;
    }

    delete EndSessionDialog.EndSessionDialog.prototype._onHibernate;
    delete EndSessionDialog.EndSessionDialog.prototype._onSuspend;
    EndSessionDialog.EndSessionDialog.prototype._updateButtons =
        origUpdateButtons;
    EndSessionDialog.DialogContent[1] = originalDialogContent;

    usermenu._updateSuspendOrPowerOff = origUpdateSuspendOrPowerOff;
    usermenu._updateHaveSuspend();
    usermenu._updateSuspendOrPowerOff();
}
